# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/reclm',
                'doc_host': 'https://AnswerDotAI.github.io',
                'git_url': 'https://github.com/AnswerDotAI/reclm',
                'lib_path': 'reclm'},
  'syms': { 'reclm.cli': { 'reclm.cli._clean_cache': ('cli.html#_clean_cache', 'reclm/cli.py'),
                           'reclm.cli.update_cache': ('cli.html#update_cache', 'reclm/cli.py')},
            'reclm.core': { 'reclm.core._CacheLock': ('core.html#_cachelock', 'reclm/core.py'),
                            'reclm.core._CacheLock.__enter__': ('core.html#_cachelock.__enter__', 'reclm/core.py'),
                            'reclm.core._CacheLock.__exit__': ('core.html#_cachelock.__exit__', 'reclm/core.py'),
                            'reclm.core._CacheLock.__init__': ('core.html#_cachelock.__init__', 'reclm/core.py'),
                            'reclm.core._RecLMAsyncClient': ('core.html#_reclmasyncclient', 'reclm/core.py'),
                            'reclm.core._RecLMAsyncClient.__init__': ('core.html#_reclmasyncclient.__init__', 'reclm/core.py'),
                            'reclm.core._RecLMAsyncClient._hash': ('core.html#_reclmasyncclient._hash', 'reclm/core.py'),
                            'reclm.core._RecLMAsyncClient._req_data': ('core.html#_reclmasyncclient._req_data', 'reclm/core.py'),
                            'reclm.core._RecLMAsyncClient.send': ('core.html#_reclmasyncclient.send', 'reclm/core.py'),
                            'reclm.core._RecLMClient': ('core.html#_reclmclient', 'reclm/core.py'),
                            'reclm.core._RecLMClient.__init__': ('core.html#_reclmclient.__init__', 'reclm/core.py'),
                            'reclm.core._RecLMClient._hash': ('core.html#_reclmclient._hash', 'reclm/core.py'),
                            'reclm.core._RecLMClient._req_data': ('core.html#_reclmclient._req_data', 'reclm/core.py'),
                            'reclm.core._RecLMClient.send': ('core.html#_reclmclient.send', 'reclm/core.py'),
                            'reclm.core._cache_dir': ('core.html#_cache_dir', 'reclm/core.py'),
                            'reclm.core._mk_patch': ('core.html#_mk_patch', 'reclm/core.py'),
                            'reclm.core.enable_reclm': ('core.html#enable_reclm', 'reclm/core.py')}}}
